package cn.com.qmth.examcloud.service.core.service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.common.util.excel.ExcelReader;
import cn.com.qmth.examcloud.common.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.service.core.dto.CourseDTO;
import cn.com.qmth.examcloud.service.core.entity.Course;
import cn.com.qmth.examcloud.service.core.enums.CourseLevel;
import cn.com.qmth.examcloud.service.core.repo.CourseRepo;

@Service
public class CourseService {

    @Autowired
    CourseRepo courseRepo;

	public List<ExcelError> importCourse(Long orgId,InputStream inputStream) {
		List<Course> list = new ArrayList<Course>();
		ExcelReader excelReader = new ExcelReader(CourseDTO.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
            	CourseDTO dto = (CourseDTO) obj;
            	Course course = new Course(dto.getName(),dto.getCode());
            	course.setOrgId(orgId);
            	if("0".equalsIgnoreCase(dto.getLevel())){
            		course.setLevel(CourseLevel.ZSB);
            	}
            	if("1".equalsIgnoreCase(dto.getLevel())){
            		course.setLevel(CourseLevel.GQZ);
            	}
                ExcelError error = importCheck(course);
                if (error == null) {
                	list.add(course);
                }
                return error;
            }
        });
        courseRepo.save(list);
		return excelErrors;
	}

	private ExcelError importCheck(Course course) {
		if(null == course.getLevel()){
			return new ExcelError("层次不存在");
		}
		Course domain = courseRepo.findByOrgIdAndCode(course.getOrgId(),course.getCode());
		if(domain != null){
			return new ExcelError("代码已存在");
		}
		return null;
	}
	
	public Page<Course> findAll(Course orgCriteria, Pageable pageable) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", startsWith())
				.withMatcher("code", startsWith()).withIgnoreNullValues();
		Example<Course> examExamStudentple = Example.of(orgCriteria,
				exampleMatcher);
		return courseRepo.findAll(examExamStudentple, pageable);
	}

	public List<Course> findAll(Course orgCriteria) {
		ExampleMatcher exampleMatcher = ExampleMatcher.matching()
				.withMatcher("name", startsWith())
				.withMatcher("code", startsWith()).withIgnoreNullValues();
        Example<Course> examExample = Example.of(orgCriteria,exampleMatcher);
		List<Course> courseList = courseRepo.findAll(examExample);
        return courseList;
	}

}
